package ru.ifmo.vizi.ctree;

import ru.ifmo.vizi.base.auto.*;
import java.util.Locale;

public final class CTree extends BaseAutomataWithListener {
    /**
      * .
      */
    public final Data d = new Data();

    /**
      *   
      */
    public CTree(Locale locale) {
        super("ru.ifmo.vizi.ctree.Comments", locale); 
        init(new Main(), d); 
    }

    /**
      * .
      */
    public final class Data {
        /**
          * Number of recursions.
          */
        public int rec;

        /**
          * Number of nodes.
          */
        public int n;

        /**
          * Matrix of weights.
          */
        public double[][] graph = null;

        /**
          * Matrix of zero edges.
          */
        public double[][] zeros = null;

        /**
          * Built tree.
          */
        public double[][] tree = null;

        /**
          * Array of minimums.
          */
        public double[] mins = null;

        /**
          * Array of parents in dfs tree.
          */
        public int[] par = null;

        /**
          * Is graph connected.
          */
        public boolean connect;

        /**
          * Is graph a tree.
          */
        public boolean isTree;

        /**
          * Strongly connected components.
          */
        public int[] scc = null;

        /**
          * Number of strongly connected components.
          */
        public int ncc;

        /**
          * Current opening component.
          */
        public int comp;

        /**
          * Hilighted component.
          */
        public int vcomp;

        /**
          * Array of the components' roots.
          */
        public int[] crts;

        /**
          * Stack of graphs.
          */
        public AutoStack grStack = new AutoStack();

        /**
          * Functions for CTree.
          */
        public CTreeFuns funs;

        /**
          * Instance of visualizer.
          */
        public CTreeVisualizer visualizer;

        public String toString() {
            StringBuffer s = new StringBuffer();
            s.append("n = " + n);
            
            return s.toString();
        }
    }

    /**
      * Builds a shortest tree of paths.
      */
    private final class Main implements Automata {
        /**
          *   .
          */
        private final int START_STATE = 0;

        /**
          *   .
          */
        private final int END_STATE = 27;

        /**
          *  .
          */
        private final String[] descriptions = new String[]{" ", "Initialization", "Test if there is path to every vertice from A", "Test if there is path to every vertice from A", "Test if there is path to every vertice from A ()", "Finding minimums", "Descend to zero", "Try to build tree", "If tree is build", "If tree is build ()", "Build graph of strongly connected components", "Creating of new graph", "Step before recursion", "Builds a shortest tree of paths ()", "Pre Backstep", "Backstep", "Build tree in first component", "Tree in the first component is built", "Build trees in opened components", "Next component", "Build trees in one component", "No more components", "Build tree in the hole graph", "Build solution tree", "test if it is first graph.", "test if it is first graph. ()", "Cannot find solution", " "};

        /**
          *   .
          */
        private int state;

        /**
          *   .
          */
        private Automata child;

        /**
          *    .
          */
        public void toStart() {
            state = START_STATE; 
            child = null; 
        }

        /**
          *    .
          */
        public void toEnd() {
            state = END_STATE; 
            child = null; 
        }

        /**
          *      .
          */
        public boolean isAtStart() {
            return state == START_STATE; 
        }

        /**
          *      .
          */
        public boolean isAtEnd() {
            return state == END_STATE; 
        }

        /**
          *   .
          */
        public int getStep() {
            return step; 
        }

        /**
          *    .
          */
        public void stepForward(int level) {
            do {
                step++; 
                //    
                switch (state) {
                    case START_STATE: { //  
                        state = 1; // Initialization
                        break;
                    }
                    case 1: { // Initialization
                        state = 2; // Test if there is path to every vertice from A
                        break;
                    }
                    case 2: { // Test if there is path to every vertice from A
                        state = 3; // Test if there is path to every vertice from A
                        break;
                    }
                    case 3: { // Test if there is path to every vertice from A
                        if (d.connect) {
                            state = 5; // Finding minimums
                        } else {
                            state = 26; // Cannot find solution
                        }
                        break;
                    }
                    case 4: { // Test if there is path to every vertice from A ()
                        state = END_STATE; 
                        break;
                    }
                    case 5: { // Finding minimums
                        state = 6; // Descend to zero
                        break;
                    }
                    case 6: { // Descend to zero
                        state = 7; // Try to build tree
                        break;
                    }
                    case 7: { // Try to build tree
                        state = 8; // If tree is build
                        break;
                    }
                    case 8: { // If tree is build
                        if (!d.isTree) {
                            state = 10; // Build graph of strongly connected components
                        } else {
                            state = 23; // Build solution tree
                        }
                        break;
                    }
                    case 9: { // If tree is build ()
                        state = 24; // test if it is first graph.
                        break;
                    }
                    case 10: { // Build graph of strongly connected components
                        state = 11; // Creating of new graph
                        break;
                    }
                    case 11: { // Creating of new graph
                        state = 12; // Step before recursion
                        break;
                    }
                    case 12: { // Step before recursion
                        state = 13; // Builds a shortest tree of paths ()
                        break;
                    }
                    case 13: { // Builds a shortest tree of paths ()
                        if (child.isAtEnd()) {
                            child = null; 
                            state = 14; // Pre Backstep
                        }
                        break;
                    }
                    case 14: { // Pre Backstep
                        state = 15; // Backstep
                        break;
                    }
                    case 15: { // Backstep
                        state = 16; // Build tree in first component
                        break;
                    }
                    case 16: { // Build tree in first component
                        state = 17; // Tree in the first component is built
                        break;
                    }
                    case 17: { // Tree in the first component is built
                        stack.pushBoolean(false); 
                        state = 18; // Build trees in opened components
                        break;
                    }
                    case 18: { // Build trees in opened components
                        if (d.comp < d.ncc) {
                            state = 19; // Next component
                        } else {
                            state = 21; // No more components
                        }
                        break;
                    }
                    case 19: { // Next component
                        state = 20; // Build trees in one component
                        break;
                    }
                    case 20: { // Build trees in one component
                        stack.pushBoolean(true); 
                        state = 18; // Build trees in opened components
                        break;
                    }
                    case 21: { // No more components
                        state = 22; // Build tree in the hole graph
                        break;
                    }
                    case 22: { // Build tree in the hole graph
                        stack.pushBoolean(true); 
                        state = 9; // If tree is build ()
                        break;
                    }
                    case 23: { // Build solution tree
                        stack.pushBoolean(false); 
                        state = 9; // If tree is build ()
                        break;
                    }
                    case 24: { // test if it is first graph.
                        if (d.rec == 0) {
                            stack.pushBoolean(true); 
                            state = 25; // test if it is first graph. ()
                        } else {
                            stack.pushBoolean(false); 
                            state = 25; // test if it is first graph. ()
                        }
                        break;
                    }
                    case 25: { // test if it is first graph. ()
                        stack.pushBoolean(true); 
                        state = 4; // Test if there is path to every vertice from A ()
                        break;
                    }
                    case 26: { // Cannot find solution
                        stack.pushBoolean(false); 
                        state = 4; // Test if there is path to every vertice from A ()
                        break;
                    }
                }

                //    
                switch (state) {
                    case 1: { // Initialization
                        	d.funs.init();
                        break;
                    }
                    case 2: { // Test if there is path to every vertice from A
                                        boolean[] visit = new boolean[d.n];
                        				d.funs.dfs(d.graph, 0, visit, null, null, 0, d.par, null, -1, 0);
                        				d.connect = true;
                        				for (int i = 0; i < d.n; i++) d.connect &= visit[i];
                        break;
                    }
                    case 3: { // Test if there is path to every vertice from A
                        break;
                    }
                    case 4: { // Test if there is path to every vertice from A ()
                        break;
                    }
                    case 5: { // Finding minimums
                        		for (int j = 0; j < d.n; j++)
                            	{
                            	    d.mins[j] = Double.POSITIVE_INFINITY;
                            		for (int i = 0; i < d.n; i++)
                            		{
                            			if (d.graph[i][j] < d.mins[j]) d.mins[j] = d.graph[i][j];
                            		}
                            	}
                        break;
                    }
                    case 6: { // Descend to zero
                           		for (int j = 0; j < d.n; j++)
                        		{
                        			for (int i = 0; i < d.n; i++) d.graph[i][j] -= d.mins[j];
                        		}
                        break;
                    }
                    case 7: { // Try to build tree
                                boolean[] visit = new boolean[d.n];
                           		for (int i = 0; i < d.n; i++)
                        		{
                        			for (int j = 0; j < d.n; j++)
                        			{
                        				if (d.graph[i][j] == 0) d.zeros[i][j] = 0;
                        				else d.zeros[i][j] = Double.NaN;
                        			}
                        		}
                        
                        		for (int i = 0; i < d.n; i++) d.par[i] = -1;
                        		d.funs.dfs(d.zeros, 0, visit, null, null, 0, d.par, null, -1, 0);
                        		d.isTree = true;
                        		for (int i = 0; i < d.n; i++) d.isTree &= visit[i];
                        break;
                    }
                    case 8: { // If tree is build
                        break;
                    }
                    case 9: { // If tree is build ()
                        break;
                    }
                    case 10: { // Build graph of strongly connected components
                        d.funs.pushArrayInt(stack, d.par);
                        d.ncc = d.funs.scc(d.zeros, d.scc);
                        d.funs.renum(d.scc);
                        break;
                    }
                    case 11: { // Creating of new graph
                                                        d.funs.pushArrayInt(d.grStack, d.par);
                                                        d.funs.pushArray2(d.grStack, d.graph);
                        								d.funs.pushArray2(d.grStack, d.zeros);
                                                        d.funs.pushArray2(d.grStack, d.tree);
                                                        d.funs.pushArray1(d.grStack, d.mins);
                                                        d.funs.pushArrayInt(d.grStack, d.scc);
                                                        d.grStack.pushInteger(d.ncc);
                                                        d.grStack.pushInteger(d.n);
                        
                                                        d.visualizer.copyGraph(true);
                        
                                                   		double[][] gr = new double[d.ncc][d.ncc];
                                                		for (int i = 0; i < d.ncc; i++)
                                                		{
                                                			for (int j = 0; j < d.ncc; j++)
                                                			{
                                                				gr[i][j] = Double.POSITIVE_INFINITY;
                                                			}
                                                		}
                        
                                                		for (int i = 0; i < d.n; i++)
                                                		{
                                                			for (int j = 0; j < d.n; j++)
                                                			{
                                                				if (d.scc[i] == d.scc[j]) gr[d.scc[i]][d.scc[j]] = Double.NaN;
                                                				else if (d.graph[i][j] < gr[d.scc[i]][d.scc[j]])
                                                				{
                                                					gr[d.scc[i]][d.scc[j]] = d.graph[i][j];
                                                				}
                                                			}
                                                		}
                        
                                                		for (int i = 0; i < d.ncc; i++)
                                                		{
                                                			for (int j = 0; j < d.ncc; j++)
                                                			{
                                                				if (Double.isInfinite(gr[i][j]))
                                                				{
                                                					gr[i][j] = Double.NaN;
                                                				}
                                                			}
                                                		}
                        
                                                        d.graph = gr;
                                                        d.n = d.ncc;
                                                        d.visualizer.setCollapse(true);
                                                        d.visualizer.reshapePane();
                        break;
                    }
                    case 12: { // Step before recursion
                        d.visualizer.clearGraph();
                        d.visualizer.reshapePane();
                        d.rec++;
                        break;
                    }
                    case 13: { // Builds a shortest tree of paths ()
                        if (child == null) {
                            child = new Main(); 
                            child.toStart(); 
                        }
                        child.stepForward(level); 
                        step--; 
                        break;
                    }
                    case 14: { // Pre Backstep
                        d.rec--;
                        break;
                    }
                    case 15: { // Backstep
                                                        d.funs.pushArrayInt(stack, d.par);
                                                        d.funs.pushArray2(stack, d.graph);
                        								d.funs.pushArray2(stack, d.zeros);
                                                        d.funs.pushArray2(stack, d.tree);
                                                        d.funs.pushArray1(stack, d.mins);
                                                        d.funs.pushArrayInt(stack, d.scc);
                                                        if (d.crts != null)
                                                        {
                        	                                d.funs.pushArrayInt(stack, d.crts);
                        	                                stack.pushBoolean(true);
                        	                            }
                        	                            else stack.pushBoolean(false);
                                                        stack.pushInteger(d.ncc);
                                                        stack.pushInteger(d.n);
                                                        stack.pushInteger(d.comp);
                                                        stack.pushInteger(d.vcomp);
                        
                                                        d.visualizer.copyGraph(true);
                        
                                                        double[][] grt = d.tree;
                                                        boolean[] visit = new boolean[d.n];
                        
                                                        d.n = d.grStack.popInteger();
                                                        d.ncc = d.grStack.popInteger();
                        
                                                        d.par = new int[d.n];
                                                        d.scc = new int[d.n];
                                                        d.crts = null;
                                                        d.funs.popArrayInt(d.grStack, d.scc);
                        
                                                        d.mins = new double[d.n];
                                                        d.funs.popArray1(d.grStack, d.mins);
                        
                                                        d.tree = new double[d.n][d.n];
                                                        d.funs.popArray2(d.grStack, d.tree);
                        
                                                        d.zeros = new double[d.n][d.n];
                                                        d.funs.popArray2(d.grStack, d.zeros);
                        
                                                        d.graph = new double[d.n][d.n];
                                                        d.funs.popArray2(d.grStack, d.graph);
                        
                                                        d.par = new int[d.n];
                                                        d.funs.popArrayInt(d.grStack, d.par);
                        
                                                        d.funs.pushArray2(stack, d.tree);
                                                        d.funs.pushArray2(stack, d.zeros);
                                                        d.funs.pushArrayInt(stack, d.par);
                        
                                                   		for (int i = 0; i < d.n; i++)
                                                		{
                                                			for (int j = 0; j < d.n; j++)
                                                			{
                                                				if (d.scc[i] != d.scc[j])
                                                				{
                                                					d.zeros[i][j] = Double.NaN;
                                                				}
                                                			}
                                                		}
                                                		for (int i = 0; i < d.n; i++) d.par[i] = -1;
                                                		d.crts = new int[d.ncc];
                                                		boolean[] grv = new boolean[d.ncc];
                                                		//grv[0] = true;
                                                		d.crts[0] = 0;
                                                		for (int i = 0; i < d.n; i++)
                                                		{
                                                			for (int j = 0; j < d.n; j++)
                                                			{
                                                				if (!grv[d.scc[j]] && (d.scc[i] != d.scc[j]) &&
                                                					(grt[d.scc[i]][d.scc[j]] == d.graph[i][j]))
                                                				{
                                                					d.tree[i][j] = d.graph[i][j];
                                                					grv[d.scc[j]] = true;
                                                					d.crts[d.scc[j]] = j;
                                                				}
                                                				else d.tree[i][j] = d.zeros[i][j];
                                                			}
                                                		}
                        
                                                        d.visualizer.setCollapse(false);
                                                        d.visualizer.reshapePane();
                        break;
                    }
                    case 16: { // Build tree in first component
                            d.comp = 0;
                        	d.vcomp = 0;
                        break;
                    }
                    case 17: { // Tree in the first component is built
                                                        d.funs.pushArray2(stack, d.tree);
                                                        d.visualizer.clearGraph();
                                                        d.visualizer.reshapePane();
                        
                                               			boolean[] visit = new boolean[d.n];
                                               			d.funs.dfs(d.zeros, d.crts[d.comp], visit, null, null, 0, d.par, null, -1, 0);
                        								d.comp++;
                        break;
                    }
                    case 18: { // Build trees in opened components
                        break;
                    }
                    case 19: { // Next component
                        	d.vcomp++;
                        break;
                    }
                    case 20: { // Build trees in one component
                                                            d.funs.pushArray2(stack, d.tree);
                        
                                                   			boolean[] visit = new boolean[d.n];
                                                   			d.funs.dfs(d.zeros, d.crts[d.comp], visit, null, null, 0, d.par, null, -1, 0);
                        									d.comp++;
                        break;
                    }
                    case 21: { // No more components
                        	d.vcomp++;
                        break;
                    }
                    case 22: { // Build tree in the hole graph
                                d.funs.pushArray2(stack, d.tree);
                        
                           		for (int i = 0; i < d.n; i++)
                        		{
                        			for (int j = 0; j < d.n; j++)
                        			{
                        				if ((d.scc[i] == d.scc[j]) && (i != d.par[j]))
                        				{
                        					d.tree[i][j] = Double.NaN;
                        				}
                        			}
                        		}
                        
                        		d.funs.restore(d.tree, d.mins);
                        break;
                    }
                    case 23: { // Build solution tree
                                    d.funs.pushArray2(stack, d.graph);
                                    d.funs.pushArray2(stack, d.tree);
                           			d.funs.restore(d.graph, d.mins);
                        			for (int i = 0; i < d.n; i++)
                        			{
                        				for (int j = 0; j < d.n; j++)
                        				{
                        					if (d.par[j] == i) d.tree[i][j] = d.graph[i][j];
                        					else d.tree[i][j] = Double.NaN;
                        				}
                        			}
                        break;
                    }
                    case 24: { // test if it is first graph.
                        break;
                    }
                    case 25: { // test if it is first graph. ()
                        break;
                    }
                    case 26: { // Cannot find solution
                        break;
                    }
                }
            } while (!isInteresting(level));
        }

        /**
          *    .
          */
        public void stepBackward(int level) {
            do {
                //     
                switch (state) {
                    case 1: { // Initialization
                        break;
                    }
                    case 2: { // Test if there is path to every vertice from A
                        break;
                    }
                    case 3: { // Test if there is path to every vertice from A
                        break;
                    }
                    case 4: { // Test if there is path to every vertice from A ()
                        break;
                    }
                    case 5: { // Finding minimums
                        for (int i=0; i < d.n; i++)
                        {
                            d.mins[i] = 0;
                        }
                        break;
                    }
                    case 6: { // Descend to zero
                        for (int j = 0; j < d.n; j++)
                        {
                            for (int i = 0; i < d.n; i++) d.graph[i][j] += d.mins[j];
                        }
                        break;
                    }
                    case 7: { // Try to build tree
                        d.isTree = false;
                        break;
                    }
                    case 8: { // If tree is build
                        break;
                    }
                    case 9: { // If tree is build ()
                        break;
                    }
                    case 10: { // Build graph of strongly connected components
                        d.ncc = 0;
                        d.funs.popArrayInt(stack, d.par);
                        break;
                    }
                    case 11: { // Creating of new graph
                        d.n = d.grStack.popInteger();
                        d.ncc = d.grStack.popInteger();
                        
                        d.crts = null;
                        
                        d.scc = new int[d.n];
                        d.funs.popArrayInt(d.grStack, d.scc);
                        
                        d.mins = new double[d.n];
                        d.funs.popArray1(d.grStack, d.mins);
                        
                        d.tree = new double[d.n][d.n];
                        d.funs.popArray2(d.grStack, d.tree);
                        
                        d.zeros = new double[d.n][d.n];
                        d.funs.popArray2(d.grStack, d.zeros);
                        
                        d.graph = new double[d.n][d.n];
                        d.funs.popArray2(d.grStack, d.graph);
                        
                        d.par = new int[d.n];
                        d.funs.popArrayInt(d.grStack, d.par);
                        d.visualizer.popGraph();
                        d.visualizer.reshapePane();
                        break;
                    }
                    case 12: { // Step before recursion
                        d.visualizer.copyGraph(false);
                        d.visualizer.reshapePane();
                        d.rec--;
                        break;
                    }
                    case 13: { // Builds a shortest tree of paths ()
                        if (child == null) {
                            child = new Main(); 
                            child.toEnd(); 
                        }
                        child.stepBackward(level); 
                        step++; 
                        break;
                    }
                    case 14: { // Pre Backstep
                        d.rec++;
                        break;
                    }
                    case 15: { // Backstep
                                                        d.funs.popArrayInt(stack, d.par);
                        								d.funs.popArray2(stack, d.zeros);
                                                        d.funs.popArray2(stack, d.tree);
                                                        d.funs.pushArrayInt(d.grStack, d.par);
                                                        d.funs.pushArray2(d.grStack, d.graph);
                                                        d.funs.pushArray2(d.grStack, d.zeros);
                                                        d.funs.pushArray2(d.grStack, d.tree);
                                                        d.funs.pushArray1(d.grStack, d.mins);
                                                        d.funs.pushArrayInt(d.grStack, d.scc);
                                                        d.grStack.pushInteger(d.ncc);
                                                        d.grStack.pushInteger(d.n);
                        
                                                        d.vcomp = stack.popInteger();
                                                        d.comp = stack.popInteger();
                                                        d.n = stack.popInteger();
                                                        d.ncc = stack.popInteger();
                                                        boolean b = stack.popBoolean();
                        
                                                        d.scc = new int[d.n];
                                                        if (b)
                                                        {
                        	                                d.crts = new int[d.ncc];
                        									d.funs.popArrayInt(stack, d.crts);
                        								}
                        								else d.crts = null;
                                                        d.funs.popArrayInt(stack, d.scc);
                        
                                                        d.mins = new double[d.n];
                                                        d.funs.popArray1(stack, d.mins);
                        
                                                        d.tree = new double[d.n][d.n];
                                                        d.funs.popArray2(stack, d.tree);
                        
                                                        d.zeros = new double[d.n][d.n];
                                                        d.funs.popArray2(stack, d.zeros);
                        
                                                        d.graph = new double[d.n][d.n];
                                                        d.funs.popArray2(stack, d.graph);
                        
                                                        d.par = new int[d.n];
                                                        d.funs.popArrayInt(stack, d.par);
                                                        d.visualizer.popGraph();
                                                        d.visualizer.reshapePane();
                        break;
                    }
                    case 16: { // Build tree in first component
                        break;
                    }
                    case 17: { // Tree in the first component is built
                        d.visualizer.copyGraph(false);
                        d.visualizer.reshapePane();
                        d.funs.popArray2(stack, d.tree);
                        break;
                    }
                    case 18: { // Build trees in opened components
                        break;
                    }
                    case 19: { // Next component
                        	d.vcomp--;
                        break;
                    }
                    case 20: { // Build trees in one component
                        d.funs.popArray2(stack, d.tree);
                        d.comp--;
                        break;
                    }
                    case 21: { // No more components
                        	d.vcomp--;
                        break;
                    }
                    case 22: { // Build tree in the hole graph
                        d.funs.popArray2(stack, d.tree);
                        break;
                    }
                    case 23: { // Build solution tree
                        d.funs.popArray2(stack, d.tree);
                        d.funs.popArray2(stack, d.graph);
                        break;
                    }
                    case 24: { // test if it is first graph.
                        break;
                    }
                    case 25: { // test if it is first graph. ()
                        break;
                    }
                    case 26: { // Cannot find solution
                        break;
                    }
                }

                //    
                switch (state) {
                    case 1: { // Initialization
                        state = START_STATE; 
                        break;
                    }
                    case 2: { // Test if there is path to every vertice from A
                        state = 1; // Initialization
                        break;
                    }
                    case 3: { // Test if there is path to every vertice from A
                        state = 2; // Test if there is path to every vertice from A
                        break;
                    }
                    case 4: { // Test if there is path to every vertice from A ()
                        if (stack.popBoolean()) {
                            state = 25; // test if it is first graph. ()
                        } else {
                            state = 26; // Cannot find solution
                        }
                        break;
                    }
                    case 5: { // Finding minimums
                        state = 3; // Test if there is path to every vertice from A
                        break;
                    }
                    case 6: { // Descend to zero
                        state = 5; // Finding minimums
                        break;
                    }
                    case 7: { // Try to build tree
                        state = 6; // Descend to zero
                        break;
                    }
                    case 8: { // If tree is build
                        state = 7; // Try to build tree
                        break;
                    }
                    case 9: { // If tree is build ()
                        if (stack.popBoolean()) {
                            state = 22; // Build tree in the hole graph
                        } else {
                            state = 23; // Build solution tree
                        }
                        break;
                    }
                    case 10: { // Build graph of strongly connected components
                        state = 8; // If tree is build
                        break;
                    }
                    case 11: { // Creating of new graph
                        state = 10; // Build graph of strongly connected components
                        break;
                    }
                    case 12: { // Step before recursion
                        state = 11; // Creating of new graph
                        break;
                    }
                    case 13: { // Builds a shortest tree of paths ()
                        if (child.isAtStart()) {
                            child = null; 
                            state = 12; // Step before recursion
                        }
                        break;
                    }
                    case 14: { // Pre Backstep
                        state = 13; // Builds a shortest tree of paths ()
                        break;
                    }
                    case 15: { // Backstep
                        state = 14; // Pre Backstep
                        break;
                    }
                    case 16: { // Build tree in first component
                        state = 15; // Backstep
                        break;
                    }
                    case 17: { // Tree in the first component is built
                        state = 16; // Build tree in first component
                        break;
                    }
                    case 18: { // Build trees in opened components
                        if (stack.popBoolean()) {
                            state = 20; // Build trees in one component
                        } else {
                            state = 17; // Tree in the first component is built
                        }
                        break;
                    }
                    case 19: { // Next component
                        state = 18; // Build trees in opened components
                        break;
                    }
                    case 20: { // Build trees in one component
                        state = 19; // Next component
                        break;
                    }
                    case 21: { // No more components
                        state = 18; // Build trees in opened components
                        break;
                    }
                    case 22: { // Build tree in the hole graph
                        state = 21; // No more components
                        break;
                    }
                    case 23: { // Build solution tree
                        state = 8; // If tree is build
                        break;
                    }
                    case 24: { // test if it is first graph.
                        state = 9; // If tree is build ()
                        break;
                    }
                    case 25: { // test if it is first graph. ()
                        if (stack.popBoolean()) {
                            state = 24; // test if it is first graph.
                        } else {
                            state = 24; // test if it is first graph.
                        }
                        break;
                    }
                    case 26: { // Cannot find solution
                        state = 3; // Test if there is path to every vertice from A
                        break;
                    }
                    case END_STATE: { //  
                        state = 4; // Test if there is path to every vertice from A ()
                        break;
                    }
                }

                step--; 
            } while (!isInteresting(level));
        }

        /**
          *    .
          */
        public boolean isInteresting(int level) {
            // 
            switch (state) {
                case START_STATE: //  
                    return true; 
                case 1: // Initialization
                    return level <= 0; 
                case 2: // Test if there is path to every vertice from A
                    return level <= -1; 
                case 3: // Test if there is path to every vertice from A
                    return level <= -1; 
                case 4: // Test if there is path to every vertice from A ()
                    return level <= -1; 
                case 5: // Finding minimums
                    return level <= 0; 
                case 6: // Descend to zero
                    return level <= 0; 
                case 7: // Try to build tree
                    return level <= 0; 
                case 8: // If tree is build
                    return level <= 0; 
                case 9: // If tree is build ()
                    return level <= -1; 
                case 10: // Build graph of strongly connected components
                    return level <= 0; 
                case 11: // Creating of new graph
                    return level <= 0; 
                case 12: // Step before recursion
                    return level <= -1; 
                case 13: // Builds a shortest tree of paths ()
                    return (child != null) && !child.isAtEnd(); 
                case 14: // Pre Backstep
                    return level <= 0; 
                case 15: // Backstep
                    return level <= 0; 
                case 16: // Build tree in first component
                    return level <= 0; 
                case 17: // Tree in the first component is built
                    return level <= 0; 
                case 18: // Build trees in opened components
                    return level <= -1; 
                case 19: // Next component
                    return level <= 0; 
                case 20: // Build trees in one component
                    return level <= 0; 
                case 21: // No more components
                    return level <= 0; 
                case 22: // Build tree in the hole graph
                    return level <= 0; 
                case 23: // Build solution tree
                    return level <= -1; 
                case 24: // test if it is first graph.
                    return level <= 0; 
                case 25: // test if it is first graph. ()
                    return level <= -1; 
                case 26: // Cannot find solution
                    return level <= 0; 
                case END_STATE: //  
                    return true; 
            }

            throw new RuntimeException("isInterest"); 
        }

        /**
          *    
          */
        public String getComment() {
            String comment = ""; 
            Object[] args = null; 
            //  
            switch (state) {
                case 1: { // Initialization
                    comment = CTree.this.getComment("Main.Initialization"); 
                    break;
                }
                case 5: { // Finding minimums
                    comment = CTree.this.getComment("Main.prezero"); 
                    break;
                }
                case 6: { // Descend to zero
                    comment = CTree.this.getComment("Main.zero"); 
                    break;
                }
                case 7: { // Try to build tree
                    comment = CTree.this.getComment("Main.tryTree"); 
                    break;
                }
                case 8: { // If tree is build
                    if (!d.isTree) {
                        comment = CTree.this.getComment("Main.testTree.true"); 
                    } else {
                        comment = CTree.this.getComment("Main.testTree.false"); 
                    }
                    break;
                }
                case 10: { // Build graph of strongly connected components
                    comment = CTree.this.getComment("Main.SCC"); 
                    break;
                }
                case 11: { // Creating of new graph
                    comment = CTree.this.getComment("Main.newGraph"); 
                    break;
                }
                case 13: { // Builds a shortest tree of paths ()
                    comment = child.getComment(); 
                    args = new Object[0]; 
                    break;
                }
                case 14: { // Pre Backstep
                    comment = CTree.this.getComment("Main.prebackstep"); 
                    break;
                }
                case 15: { // Backstep
                    comment = CTree.this.getComment("Main.backstep"); 
                    break;
                }
                case 16: { // Build tree in first component
                    comment = CTree.this.getComment("Main.rootComponent"); 
                    break;
                }
                case 17: { // Tree in the first component is built
                    comment = CTree.this.getComment("Main.rootComponentTree"); 
                    break;
                }
                case 19: { // Next component
                    comment = CTree.this.getComment("Main.whileTrue"); 
                    break;
                }
                case 20: { // Build trees in one component
                    comment = CTree.this.getComment("Main.preBuildtree"); 
                    args = new Object[]{new Character((char) ('A' + d.crts[d.comp - 1]))}; 
                    break;
                }
                case 21: { // No more components
                    comment = CTree.this.getComment("Main.whileFalse"); 
                    break;
                }
                case 22: { // Build tree in the hole graph
                    comment = CTree.this.getComment("Main.buildtree"); 
                    break;
                }
                case 24: { // test if it is first graph.
                    if (d.rec == 0) {
                        comment = CTree.this.getComment("Main.testRec.true"); 
                    } else {
                        comment = CTree.this.getComment("Main.testRec.false"); 
                    }
                    break;
                }
                case 26: { // Cannot find solution
                    comment = CTree.this.getComment("Main.error"); 
                    break;
                }
            }

            return java.text.MessageFormat.format(comment, args); 
        }

        /**
          *     
          */
        public void drawState() {
            switch (state) {
                case 1: { // Initialization
                    d.visualizer.updatePane(d.visualizer.STANDARD);
                    break;
                }
                case 5: { // Finding minimums
                    d.visualizer.updatePane(d.visualizer.PREZERO);
                    break;
                }
                case 6: { // Descend to zero
                    d.visualizer.updatePane(d.visualizer.ZERO);
                    break;
                }
                case 7: { // Try to build tree
                    d.visualizer.updatePane(d.visualizer.TRY_TREE);
                    break;
                }
                case 8: { // If tree is build
                    d.visualizer.updatePane(d.visualizer.TRY_TREE);
                    break;
                }
                case 10: { // Build graph of strongly connected components
                    d.visualizer.updatePane(d.visualizer.BUILD_SCC);
                    break;
                }
                case 11: { // Creating of new graph
                    d.visualizer.updatePane(d.visualizer.NEW_GRAPH);
                    break;
                }
                case 13: { // Builds a shortest tree of paths ()
                    child.drawState(); 
                    break;
                }
                case 14: { // Pre Backstep
                    d.visualizer.updatePane(d.visualizer.PRE_BUILD);
                    break;
                }
                case 15: { // Backstep
                    d.visualizer.updatePane(d.visualizer.OPEN_COMPONENTS);
                    break;
                }
                case 16: { // Build tree in first component
                    d.visualizer.updatePane(d.visualizer.PRE_TREE);
                    break;
                }
                case 17: { // Tree in the first component is built
                    	d.visualizer.updatePane(d.visualizer.PRE_TREE);
                    break;
                }
                case 19: { // Next component
                    d.visualizer.updatePane(d.visualizer.PRE_TREE);
                    break;
                }
                case 20: { // Build trees in one component
                    d.visualizer.updatePane(d.visualizer.PRE_TREE);
                    break;
                }
                case 21: { // No more components
                    d.visualizer.updatePane(d.visualizer.PRE_TREE);
                    break;
                }
                case 22: { // Build tree in the hole graph
                    d.visualizer.updatePane(d.visualizer.BUILD_TREE);
                    break;
                }
                case 24: { // test if it is first graph.
                    d.visualizer.updatePane(d.visualizer.FINISH);
                    break;
                }
                case 26: { // Cannot find solution
                    d.visualizer.updatePane(d.visualizer.ERROR);
                    break;
                }
            }
        }

        public StringBuffer toString(StringBuffer s) {
            s.append("Main ").append(state).append(" "); 
            s.append('('); 
            s.append(descriptions[state]); 
            s.append(")\n"); 
            if (child != null && !child.isAtStart() && !child.isAtEnd()) {
                child.toString(s); 
            }
            return s; 
        }
    }
}
