package ru.ifmo.vizi.ctree;

import ru.ifmo.vizi.base.auto.AutoStack;
import java.util.Stack;

public class CTreeFuns
{
    private CTree.Data d;

    public CTreeFuns(CTree ctree)
    {
        d = ctree.d;
    }

    public void init()
    {
        if (d.graph == null)
        {
            d.graph = new double[d.n][d.n];
            for (int i = 0; i < d.n; i++)
            {
               for (int j = 0; j < d.n; j++)
               {
                   d.graph[i][j] = Double.NaN;
               }
            }
        }

        d.tree = new double[d.n][d.n];
        for (int i = 0; i < d.n; i++)
        {
            for (int j = 0; j < d.n; j++)
            {
                d.tree[i][j] = Double.NaN;
            }
        }

        d.zeros = new double[d.n][d.n];

        d.mins = new double[d.n];
        d.scc = new int[d.n];
        d.par = new int[d.n];
        for (int i = 0; i < d.n; i++)
        {
        	d.par[i] = -1;
        }

        d.ncc = 0;
    }

	public int dfs(double[][] m, int v,
						   boolean[] visit, int[] fin, int[] scc, int ncc,
						   int[] par, int[] vind, int nvind, int time)
	{
	    int n = m.length;
	    int t = time;
	    visit[v] = true;
	    if (scc != null) scc[v] = ncc;
		for (int ii = 0; ii < n; ii++)
		{
		    int i = ii;
		    if ((vind != null) && (nvind < 0)) i = vind[ii];
		    if (visit[i]) continue;
			if (!Double.isNaN(m[v][i]))
			{
			    if (par != null) par[i] = v;
				int t0 = dfs(m, i, visit, fin, scc, ncc, par, vind, nvind, t + 1);
				if (nvind >= 0) nvind += (t0 - t + 1) / 2;
				t = t0;
			}	
		}
		t++;
		if (fin != null) fin[v] = t;
		if ((vind != null) && (nvind >= 0)) vind[n - nvind] = v;
		return t;
	}

	public int scc(double[][] m, int[] scc)
	{
	    int n = m.length;
		double[][] mT = new double[n][n];
		for (int i = 0; i < n; i++)
		{
			for (int j = 0; j < n; j++)
			{
				mT[i][j] = m[j][i];
			}
		}

		int[] fin = new int[n];
		boolean[] visit = new boolean[n];
		int[] vind = new int[n];
		int nvind = 1;
		int time = 0;
		for (int i = 0; i < n; i++)
		{
			if (!visit[i])
			{
				int t = dfs(m, i, visit, fin, null, 0, null, vind, nvind, time);
				if (nvind >= 0) nvind += (t - time + 1) / 2;
				time = t + 1;
			}	
		}

		int ncc = 0;
		for (int i = 0; i < n; i++) visit[i] = false;
		for (int i = 0; i < n; i++)
		{
		    int v = vind[i];
			if (!visit[v])
			{
				dfs(mT, v, visit, null, scc, ncc, null, vind, -1, 0);
				ncc++;
			}
		}
		return ncc;
	}

	public void restore(double[][] m, double[] mins)
	{
	    int n = m.length;
		for (int j = 0; j < n; j++)
		{
			for (int i = 0; i < n; i++)
			{
				m[i][j] += mins[j];
			}
		}
	}


    public void renum(int[] scc)
    {
    	int n = scc.length;
    	int[] perm = new int[n];
    	for (int i = 0; i < n; i++) perm[i] = -1;
    	int m = 0;

		for (int i = 0; i < n; i++)
		{
			if (perm[scc[i]] < 0) perm[scc[i]] = m++;
			scc[i] = perm[scc[i]];
		}
	}	

    public void pushArray1(AutoStack st, double[] array)
    {
        for (int i=0; i < array.length; i++)
        {
            st.pushDouble(array[i]);
        }
    }

     public void pushArray2(AutoStack st, double[][] array)
     {
        for (int i=0; i < array.length; i++)
        {
            for (int j=0; j < array.length; j++)
            {
                st.pushDouble(array[i][j]);
            }
        }
    }

    public void popArray1(AutoStack st, double[] array)
    {
        for (int i = array.length - 1; i >= 0; i--)
        {
            array[i] = st.popDouble();
        }
    }

    public void popArray2(AutoStack st, double[][] array)
    {
        for (int i = array.length - 1; i >= 0; i--)
        {
            for (int j = array.length - 1; j >= 0; j--)
            {
                array[i][j] = st.popDouble();
            }
        }
    }

    public void pushArrayInt(AutoStack st, int[] array)
    {
        for (int i=0; i < array.length; i++)
        {
            st.pushInteger(array[i]);
        }
    }

    public void popArrayInt(AutoStack st, int[] array)
    {
        for (int i = array.length - 1; i >= 0; i--)
        {
            array[i] = st.popInteger();
        }
    }
}