package ru.ifmo.vizi.ctree;

import java.awt.*;
import java.text.*;

import ru.ifmo.vizi.base.*;
import ru.ifmo.vizi.base.ui.*;

public class EditDialog extends SaveLoadDialog
{
    private CTreeVisualizer visualizer;
    private Example example;
    private String text;
    private Configuration config;

    public EditDialog(Configuration config, CTreeVisualizer vizi, Frame forefather)
    {
        super(config, forefather);
        this.visualizer = vizi;
        this.config = config;
    }

    public boolean load(String s) throws Exception
    {
        text = s;
        if (readExample())
        {
            visualizer.addExample(example);
            visualizer.setExample(example);
            return true;
        }
        return false;
    }

    public void loadExample(Example e, int step)
    {
        StringBuffer buf = new StringBuffer();

        boolean comment = config.getBoolean("edit-frame-show-comment", false);
        if (comment) buf.append(config.getParameter("edit-frame-name-comment") + "\n");
        buf.append(config.getParameter("edit-frame-name", "Name") + " = \"" + e.getName() + "\"");
        buf.append('\n');
        String commentStr = MessageFormat.format(config.getParameter("edit-frame-vertices-comment"),
        										 new Object[]
        										 {
        										 	new Integer(visualizer.minVertices),
        										 	new Integer(visualizer.maxVertices)
        										 });
        if (comment) buf.append(commentStr + "\n");
        buf.append(config.getParameter("edit-frame-vertices", "Vertices") + " = " + e.getNumber());
        buf.append("\n\n");
        if (comment) buf.append(config.getParameter("edit-frame-matrix-comment") + "\n");
        buf.append(config.getParameter("edit-frame-matrix", "Matrix") + " = \n");

        int n = e.getNumber();
        double[][] m = e.getMatrix();
        for (int i = 0; i < n; i++)
        {
            for (int j = 0; j < n; j++)
            {
                if (Double.isNaN(m[i][j]))
                {
                    buf.append('X');
                }
                else
                {
                    buf.append(m[i][j]);
                }
                if (j != n - 1) buf.append(",\t");
            }
            buf.append(";\n");
        }

        buf.append('\n');
        if (comment) buf.append(config.getParameter("edit-frame-current-comment") + "\n");
        buf.append(config.getParameter("edit-frame-current", "Step") + " = " + step);

        text = new String(buf);
        setInitialContent(text);
        setContent(text);
		setComment("");
    }

    public boolean readExample() throws Exception
    {
        String suffix = "-" + this.getLocale().getLanguage();
        example = new Example();
        example.readExample(text, config, visualizer, suffix);
        return true;
    }


}