package ru.ifmo.vizi.ctree;

import ru.ifmo.vizi.base.*;

import java.io.*;
import java.text.*;

public class Example
{
    private int n;
    private double[][] graph;
    private String name;
    private int step;

    public Example()
    {
        n = 0;
        graph = null;
        name = null;
        step = 0;
    }

    public Example(int n, double[][] graph, String name)
    {
        this.n = n;
        this.graph = graph;
        this.name = name;
    }

    public int getNumber()
    {
        return n;
    }

    public double[][] getMatrix()
    {
        double[][] m = new double[n][n];
        for (int i = 0; i < n; i++)
        {
            for (int j = 0; j < n; j++)
            {
                m[i][j] = graph[i][j];
            }
        }
        return m;
    }

    public String getName()
    {
        return new String(name);
    }

    public int getStep()
    {
        return step;
    }

    public String readExample(String str, Configuration config, CTreeVisualizer visualizer, String suffix) throws Exception
    {
        n = 0;
        step = 0;
        graph = null;
        name = null;
        SmartTokenizer tokenizer = new SmartTokenizer(str, config);
        tokenizer.parseNumbers();
        String tok;
        try
        {
            while (tokenizer.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (tokenizer.ttype != StreamTokenizer.TT_WORD)
                {
                    Object found;
                    switch (tokenizer.ttype)
                    {
                        case SmartTokenizer.TT_NUMBER:
                            found = new Double(tokenizer.nval);
                            break;
                        default:
                            found = new Character((char) tokenizer.ttype);
                    }
                    throw new Exception(MessageFormat.format(config.getParameter("edit-frame-inv-param", ""),
                            new Object[]{new Integer(tokenizer.lineno()), found}));
                }
                tok = tokenizer.sval;
                if ((tokenizer.nextToken() != '='))
                {
                    throw new Exception(MessageFormat.format(config.getParameter("edit-frame-no-eq", ""),
                            new Object[]{new Integer(tokenizer.lineno()), tok}));
                }
                if (tok.compareTo(config.getParameter("edit-frame-name", "Name")) == 0)
                {
                    name = tokenizer.nextWord();
                }
                else if (tok.compareTo(config.getParameter("edit-frame-name", "Name") + suffix.substring(1)) == 0)
                {
                    name = tokenizer.nextWord();
                }
                else if (tok.indexOf(config.getParameter("edit-frame-name", "Name")) == 0)
                {
                    tokenizer.nextToken();
                }
                else if (tok.compareTo(config.getParameter("edit-frame-vertices", "Vertices")) == 0)
                {
                    n = tokenizer.nextInt(visualizer.minVertices, visualizer.maxVertices);
                }
                else if (tok.compareTo(config.getParameter("edit-frame-current", "Step")) == 0)
                {
                    step = tokenizer.nextInt();
                }
                else if (tok.compareTo(config.getParameter("edit-frame-matrix", "Matrix")) == 0)
                {
                    if (n == 0)
                    {
                        throw new Exception(config.getParameter("edit-frame-n-not-given", ""));
                    }
                    graph = new double[n][n];
                    tokenizer.ordinaryChar('X');
                    for (int i = 0; i < n; i++)
                    {
                        for (int j = 0; j < n; j++)
                        {
                            tokenizer.nextToken();
                            if (tokenizer.ttype == SmartTokenizer.TT_EOF)
                            {
                                throw new Exception(MessageFormat.format(config.getParameter("SmartTokenizer-UnexpectedEOF"),
                                        new Object[]{new Integer(tokenizer.lineno())}));
                            }
                            if ((tokenizer.ttype == '-') || (tokenizer.ttype == 'X'))
                            {
                                graph[i][j] = Double.NaN;
                            }
                            else if (tokenizer.ttype != StreamTokenizer.TT_NUMBER)
                            {
                                Object found;
                                switch (tokenizer.ttype)
                                {
                                    case SmartTokenizer.TT_WORD:
                                        found = tokenizer.sval;
                                        break;
                                    default:
                                        found = new Character((char) tokenizer.ttype);
                                }
                                throw new Exception(MessageFormat.format(config.getParameter("edit-frame-no-num", ""),
                                        new Object[]{new Integer(tokenizer.lineno()), found}));
                            }
                            else if ((tokenizer.nval < 0) || (tokenizer.nval > config.getInteger("graph-max-val", 999)))
                            {
                                throw new Exception(MessageFormat.format(config.getParameter("edit-frame-bounds", ""),
                                        new Object[]{new Integer(tokenizer.lineno()), new Integer(0),
                                                     new Integer(config.getInteger("graph-max-val", 999)),
                                                     new Integer((int) tokenizer.nval)}));
                            }
                            else
                            {
                                graph[i][j] = tokenizer.nval;
                            }
                            if (j < n - 1)
                            {
                                tokenizer.expect(",");
                            }
                            else if (j == n - 1)
                            {
                                tokenizer.expect(";");
                            }
                            if ((i == j) && !Double.isNaN(graph[i][j]))
                            {
                            	throw new Exception(MessageFormat.format(config.getParameter("edit-frame-diag" + suffix, ""),
                            			new Object[]{new Integer(tokenizer.lineno()), new Integer(i),
                                                     new Double(graph[i][j])}));
                            }
                        }
                    }
                    tokenizer.wordChars((int) 'X', (int) 'X');
                }
                else
                {
                    throw new Exception(MessageFormat.format(config.getParameter("edit-frame-inv-param", ""),
                            new Object[]{new Integer(tokenizer.lineno()), tok}));
                }
            }
        }
        catch (IOException e)
        {
            return "IO Exception";
        }
        return null;
    }
}