package ru.ifmo.vizi.ctree;

import ru.ifmo.vizi.ctree.widgets.*;

import ru.ifmo.vizi.base.widgets.*;
import ru.ifmo.vizi.base.Configuration;

import java.awt.*;
import java.util.Stack;
import java.text.*;

public class Graph
{
    public final static int STANDARD = 0;
    public final static int PREZERO = 11;
    public final static int ZERO = 1;
    public final static int TRY_TREE = 2;
    public final static int BUILD_SCC = 3;
    public final static int NEW_GRAPH = 4;
    public final static int PRE_BUILD = 5;
    public final static int OPEN_COMPONENTS = 6;
    public final static int PRE_TREE = 7;
    public final static int BUILD_TREE = 8;
    public final static int FINISH = 9;
    public final static int ERROR = 10;

    private final boolean archive;
    private int n;
    private int ncc;
    private double[][] graph;
    private double[][] zeros;
    private double[][] tree;
    private double[] mins;
    private int[] par;
    private int[] scc;
    private int[] crts;
    private int comp;
    private int vcomp;

    private final double arrowAngle = Math.PI / 20;

    //current sizes and position.
    private int top;
    private int left;
    private int width;
    private int height;

    private String minsString;

    // relative sizes and places of drawing items.
    private double gRadius = 0.75;
    private double vRadius = 0.25;
    private double tSize = 0.9;
    private double split = 0.5;
    private boolean gAspectStatus;
    private double gAspect;
    private boolean tAspectStatus;
    private double tAspect;

    private double minsplit;
    private double maxsplit;

    // absolute sizes and places of drawing items.
    private int graphWidth;        // 75
    private int graphHeight;
    private Point graphCenter;      // 100, 100
    private int vertexRadius;       // 15
    private Point tableXY;          // 200, 20
    private Dimension tableSize;    // 160, 160
    private int asplit;

    private Rect splitRect;
    private double oldsplit;

    private ShapeStyle[] tableStyle;
    private ShapeStyle[] vertexStyle;
    private ShapeStyle[] arrowStyle;

    private final Stack vertices = new Stack();
    private final Stack gtable = new Stack();
    private final Stack arrows = new Stack();

    public Graph(CTree auto, Configuration config, String minsString,
                 double gRadius, boolean gAspectStatus, double gAspect, boolean tAspectStatus, double tAspect,
                 double vRadius, double tSize, double split, double minsplit, double maxsplit,
                 ShapeStyle[] tableStyle, ShapeStyle[] vertexStyle, ShapeStyle[] arrowStyle)
    {
        this.minsString = minsString;
        this.gRadius = gRadius;
        this.vRadius = vRadius;
        this.tSize = tSize;
        this.split = split;
        this.minsplit = minsplit;
        this.maxsplit = maxsplit;
        this.gAspectStatus = gAspectStatus;
        this.gAspect = gAspect;
        this.tAspectStatus = tAspectStatus;
        this.tAspect = tAspect;
        this.tableStyle = tableStyle;
        this.vertexStyle = vertexStyle;
        this.arrowStyle = arrowStyle;

        archive = false;

        splitRect = new Rect(new ShapeStyle[]{new ShapeStyle(config, "splitRectStyle", tableStyle[0])});
        splitRect.setCursor(new Cursor(Cursor.W_RESIZE_CURSOR));
        updateGraph(auto);
    }

    public Graph(Graph g)
    {
        archive = true;

        n = g.n;
        ncc = g.ncc;
        graph = new double[n][n];
        for (int i=0; i < n; i++)
        {
            for (int j=0; j < n; j++)
            {
                graph[i][j] = g.graph[i][j];
            }
        }
        zeros = new double[n][n];
		for (int i=0; i < n; i++)
        {
            for (int j=0; j < n; j++)
            {
                zeros[i][j] = g.zeros[i][j];
            }
        }
        tree = new double[n][n];
        for (int i=0; i < n; i++)
        {
            for (int j=0; j < n; j++)
            {
                tree[i][j] = g.tree[i][j];
            }
        }
        par = new int[n];
        for (int i=0; i < n; i++)
        {
            par[i] = g.par[i];
        }
        scc = new int[n];
        for (int i=0; i < n; i++)
        {
            scc[i] = g.scc[i];
        }
        crts = new int[ncc];
        for (int i=0; i < ncc; i++)
        {
        	crts[i] = g.crts[i];
        }
        comp = g.comp;
        vcomp = g.vcomp;

        this.minsString = g.minsString;
        this.gRadius = g.gRadius;
        this.vRadius = g.vRadius;
        this.tSize = g.tSize;
        this.split = g.split;
        this.minsplit = g.minsplit;
        this.maxsplit = g.maxsplit;
        this.gAspectStatus = g.gAspectStatus;
        this.gAspect = g.gAspect;
        this.tAspectStatus = g.tAspectStatus;
        this.tAspect = g.tAspect;
        this.tableStyle = g.tableStyle;
        this.vertexStyle = g.vertexStyle;
        this.arrowStyle = g.arrowStyle;

        splitRect = new Rect(g.splitRect.getStyleSet());
        splitRect.setCursor(new Cursor(Cursor.W_RESIZE_CURSOR));
    }

    public void updateGraph(CTree auto)
    {
        CTree.Data data = auto.d;
        if (archive) return;
        n = data.n;
        graph = new double[n][n];
        for (int i=0; i < n; i++)
        {
            for (int j=0; j < n; j++)
            {
                graph[i][j] = data.graph[i][j];
            }
        }
        if (!auto.isAtStart())
        {
            zeros = new double[n][n];
            for (int i=0; i < n; i++)
            {
                for (int j=0; j < n; j++)
                {
                    zeros[i][j] = data.zeros[i][j];
                }
            }
            tree = new double[n][n];
            for (int i=0; i < n; i++)
            {
                for (int j=0; j < n; j++)
                {
                    tree[i][j] = data.tree[i][j];
                }
            }
            mins = new double[n];
            for (int i = 0; i < n; i++)
            {
                mins[i] = data.mins[i];
            }
            par = new int[n];
            for (int i=0; i < n; i++)
            {
                par[i] = data.par[i];
            }
            scc = new int[n];
            for (int i=0; i < n; i++)
            {
                scc[i] = data.scc[i];
            }
            comp = data.comp;
            vcomp = data.vcomp;
        }
    }

    public void reshapeGraph()
    {
        reshapeGraph(top, left, width, height);
    }

    public void reshapeGraph(int top, int left, int width, int height)
    {
        updateSizes(top, left, width, height);
        for (int i = 0; i < n; i++)
        {
            Ellipse circ;
            if (vertices.size() > i)
            {
                circ = (Ellipse) vertices.elementAt(i);
            }
            else
            {
                circ = new Ellipse(vertexStyle);
                vertices.push(circ);
            }
            double x,y;
            x = graphCenter.x - graphWidth * Math.sin(2*Math.PI / n * i);
            y = graphCenter.y - graphHeight * Math.cos(2*Math.PI / n * i);
            circ.setBounds((int) Math.round(x) - vertexRadius, (int) Math.round(y) - vertexRadius, 2 * vertexRadius, 2 * vertexRadius);
            circ.adjustFontSize();
        }
        int j = vertices.size();
        while (j > n)
        {
            vertices.pop();
            j--;
        }

        int rwidth = tableSize.width / (n + 2);
        int rheight = tableSize.height / (n + 2);
        for (int i = 1; i <= n; i++)
        {
            for (int k = 1; k <= n; k++)
            {
                Rect rect;
                if (gtable.size() > (i-1) * n + (k-1))
                {
                    rect = (Rect) gtable.elementAt((i-1) * n + (k-1));
                }
                else
                {
                    rect = new Rect(tableStyle);
                    gtable.push(rect);
                }
                int x,y;
                x = tableXY.x + rwidth * k;
                y = tableXY.y + rheight * i;
                rect.setBounds(x, y, rwidth, rheight);
                rect.adjustFontSize();
                rect.setVisible(true);
            }
        }
        for (int i = 0; i < n; i++)
        {
            Rect rect;
            if (gtable.size() > n * n + 2*i)
            {
                rect = (Rect) gtable.elementAt(n * n + 2*i);
            }
            else
            {
                rect = new Rect(tableStyle);
                gtable.push(rect);
            }
            int x,y;
            x = tableXY.x + rwidth * (i+1);
            y = tableXY.y;
            rect.setBounds(x, y, rwidth, rheight);
            rect.adjustFontSize();
            rect.setVisible(true);

            if (gtable.size() > n * n + 2*i + 1)
            {
                rect = (Rect) gtable.elementAt(n * n + 2*i + 1);
            }
            else
            {
                rect = new Rect(tableStyle);
                gtable.push(rect);
            }
            x = tableXY.x;
            y = tableXY.y + rheight * (i+1);
            rect.setBounds(x, y, rwidth, rheight);
            rect.adjustFontSize();
            rect.setVisible(true);
        }
        for (int i = 0; i < n; i++)
        {
            Rect rect;
            if (gtable.size() > n*n + 2*n + i)
            {
                rect = (Rect) gtable.elementAt(n * n + 2*n + i);
            }
            else
            {
                rect = new Rect(tableStyle);
                gtable.push(rect);
            }
            int x, y;
            x = tableXY.x + rwidth * (i + 1);
            y = tableXY.y + tableSize.height - rheight;
            rect.setBounds(x, y, rwidth, rheight);
            rect.adjustFontSize(minsString);
        }
        j = gtable.size();
        while (j > n * n + 2*n + n)
        {
            gtable.pop();
            j--;
        }

        j = 0;
        for (int i = 0; i < n; i++)
        {
            for (int k = 0; k < n; k++)
            {
                if (i == k) continue;
                Arrow arr;
                if (arrows.size() > j)
                {
                    arr = (Arrow) arrows.elementAt(j);
                }
                else
                {
                    arr = new Arrow(arrowStyle);
                    arrows.push(arr);
                }
                Ellipse c1 = (Ellipse) vertices.elementAt(i);
                Ellipse c2 = (Ellipse) vertices.elementAt(k);
                Point begin = new Point(c1.getBounds().x + vertexRadius, c1.getBounds().y + vertexRadius);
                Point end = new Point(c2.getBounds().x + vertexRadius, c2.getBounds().y + vertexRadius);
                double alpha = Math.atan2(end.y - begin.y, end.x - begin.x);
                begin.translate((int) (vertexRadius * Math.cos(alpha + arrowAngle)),
                                (int) (vertexRadius * Math.sin(alpha + arrowAngle)));
                end.translate((int) (vertexRadius * Math.cos(alpha - arrowAngle - Math.PI)),
                              (int) (vertexRadius * Math.sin(alpha - arrowAngle - Math.PI)));
                arr.setArrow(begin.x, begin.y, end.x, end.y);
                j++;
            }
        }
        j = arrows.size();
        while (j > n * (n - 1))
        {
            arrows.pop();
            j--;
        }
    }

    public void addTo(Container Pane)
    {
        for (int i = 0; i < gtable.size(); i++)
        {
            Pane.add((Rect) gtable.elementAt(i));
        }
        for (int i = 0; i < vertices.size(); i++)
        {
            Pane.add((Ellipse) vertices.elementAt(i));
        }
        for (int i = 0; i < arrows.size(); i++)
        {
            Pane.add((Arrow) arrows.elementAt(i));
        }

        Pane.add(splitRect);
    }

    public void updateTable(int flag)
    {
        DecimalFormat dformat = new DecimalFormat();
        dformat.setMaximumFractionDigits(2);
        Rect rect;
        double[][] table = graph;
        for (int i = 0; i < n; i++)
        {
            for (int j = 0; j < n; j++)
            {
                rect = (Rect) gtable.elementAt(i * n + j);
                switch (flag)
                {
                    case STANDARD:
                    case PREZERO:
                        rect.setStyle(0);
                        break;
                    case ZERO:
                        if (graph[i][j] == 0)
                        {
                            rect.setStyle(1);
                        }
                        else
                        {
                            rect.setStyle(0);
                        }
                        break;
                    case TRY_TREE:
                        if (par[j] == i)
                        {
                            rect.setStyle(2);
                        }
                        else if (graph[i][j] == 0)
                        {
                            rect.setStyle(1);
                        }
                        else
                        {
                            rect.setStyle(0);
                        }
                        break;
                    case BUILD_SCC:
                        if ((graph[i][j] == 0) && (scc[i] == scc[j]))
                        {
                            rect.setStyle(scc[i] + 5);
                        }
                        else
                        {
                            rect.setStyle(0);
                        }
                        break;
                    case NEW_GRAPH:
                        rect.setStyle(0);
                        break;
                    case PRE_BUILD:
                        table = tree;
                        rect.setStyle(0);
                        break;
                    case OPEN_COMPONENTS:
                        table = tree;
                        if (!Double.isNaN(tree[i][j]))
                        {
                            if (scc[i] == scc[j])
                            {
                                rect.setStyle(scc[i] + 5);
                            }
                            else
                            {
                                rect.setStyle(3);
                            }
                        }
                        else
                        {
                            rect.setStyle(0);
                        }
                        break;
                    case PRE_TREE:
                        table = tree;
                        if (!Double.isNaN(tree[i][j]))
                        {
                            if (scc[i] != scc[j])
                            {
                                rect.setStyle(3);
                            }
                            else if ((scc[i] < comp) && (i == par[j]))
                            {
                                rect.setStyle(2);
                            }
                            else
                            {
                                rect.setStyle(scc[i] + 5);
                            }
                        }
                        else
                        {
                            rect.setStyle(0);
                        }
                        break;
                    case BUILD_TREE:
                        table = tree;
                        if (!Double.isNaN(tree[i][j]))
                        {
                            rect.setStyle(2);
                        }
                        else
                        {
                            rect.setStyle(0);
                        }
                        break;
                    case FINISH:
                        rect.setStyle(0);
                        table = tree;
                        break;
                    case ERROR:
                        break;
                }
                if (i == j)
                {
                    rect.setMessage("X");
                    rect.setStyle(12);
                }
                else if (!Double.isNaN(table[i][j]))
                {
                    rect.setMessage(dformat.format(table[i][j]));
                }
                else
                {
                    rect.setMessage("-");
                }
                rect.adjustFontSize();
            }
        }

        char[] name = new char[]{'A'};
        Rect rect1;
        Rect rect2;
        for (int i = 0; i < n; i++)
        {
            name[0] = (char) ('A' + i);
            rect1 = (Rect) gtable.elementAt(n * n + 2*i);
            rect1.setMessage(new String(name));
            rect1.adjustFontSize();

            rect2 = (Rect) gtable.elementAt(n * n + 2*i + 1);
            rect2.setMessage(new String(name));
            rect2.adjustFontSize();

            switch (flag)
            {
                case STANDARD:
                case PREZERO:
                    rect1.setStyle(13);
                    rect2.setStyle(13);
                    break;
                case ZERO:
                    rect1.setStyle(13);
                    rect2.setStyle(13);
                    break;
                case TRY_TREE:
                    if (i == 0)
                    {
                        rect1.setStyle(14);
                        rect2.setStyle(14);
                    }
                    else if (par[i] != -1)
                    {
                        rect1.setStyle(14);
                        rect2.setStyle(14);
                    }
                    else
                    {
                        rect1.setStyle(13);
                        rect2.setStyle(13);
                    }
                    break;
                case BUILD_SCC:
                    rect1.setStyle(scc[i] + 16);
                    rect2.setStyle(scc[i] + 16);
                    break;
                case NEW_GRAPH:
                    rect1.setStyle(i + 16);
                    rect2.setStyle(i + 16);
                    break;
                case PRE_BUILD:
                    rect1.setStyle(i + 16);
                    rect2.setStyle(i + 16);
                    break;
                case OPEN_COMPONENTS:
                case BUILD_TREE:
                    rect1.setStyle(scc[i] + 16);
                    rect2.setStyle(scc[i] + 16);
                    break;
                case PRE_TREE:
                    if (scc[i] == vcomp)
                    {
	                	rect1.setStyle(25);
						rect2.setStyle(25);
	                }
	                else
	                {
	                	rect1.setStyle(scc[i] + 16);
	                	rect2.setStyle(scc[i] + 16);
	                }	
                	break;
                case FINISH:
                    rect1.setStyle(13);
                    rect2.setStyle(13);
                    break;
                case ERROR:
                    if (i == 0)
                    {
                        rect1.setStyle(13);
                        rect2.setStyle(13);
                    }
                    else if (par[i] == -1)
                    {
                        rect1.setStyle(23);
                        rect2.setStyle(23);
                    }
                    else
                    {
                        rect1.setStyle(13);
                        rect2.setStyle(13);
                    }
            }
        }
        if (flag == PREZERO)
        {
            for (int i = 0; i < n; i++)
            {
                rect = (Rect) gtable.elementAt(n*n + 2*n + i);
                rect.setStyle(24);
                if (i == 0) rect.setMessage("-");
                else rect.setMessage(Double.toString(mins[i]));
                rect.setVisible(true);
            }
        }
        else
        {
            for (int i = 0; i < n; i++)
            {
                rect = (Rect) gtable.elementAt(n*n + 2*n + i);
                rect.setStyle(24);
                rect.setMessage("");
            }
        }
        updateArrows(flag);
        updateVertex(flag);
    }

    public void updateVertex(int flag)
    {
        Ellipse circ;
        char[] name = new char[]{'A'};
        for (int i = 0; i < n; i++)
        {
            circ = (Ellipse) vertices.elementAt(i);
            name[0] = (char) ('A' + i);
            switch (flag)
            {
                case STANDARD:
                case PREZERO:
                    circ.setStyle(0);
                    break;
                case ZERO:
                    circ.setStyle(0);
                    break;
                case TRY_TREE:
                    if (i == 0) circ.setStyle(1);
                    else if (par[i] != -1)
                    {
                        circ.setStyle(1);
                    }
                    else
                    {
                        circ.setStyle(0);
                    }
                    break;
                case BUILD_SCC:
                    circ.setStyle(scc[i] + 3);
                    break;
                case NEW_GRAPH:
                    circ.setStyle(i + 3);
                    break;
                case PRE_BUILD:
                    circ.setStyle(i + 3);
                    break;
                case OPEN_COMPONENTS:
                case BUILD_TREE:
                    circ.setStyle(scc[i] + 3);
                    break;
				case PRE_TREE:
					if (scc[i]	== vcomp)
					{
						circ.setStyle(11);
					}
					else
					{
						circ.setStyle(scc[i] + 3);
					}
					break;
                case FINISH:
                    circ.setStyle(0);
                    break;
                case ERROR:
                    if (i == 0) circ.setStyle(0);
                    else if (par[i] == -1)
                    {
                        circ.setStyle(10);
                    }
                    else
                    {
                        circ.setStyle(0);
                    }
            }
            circ.setMessage(new String(name));
            circ.adjustFontSize();
        }
    }

    public void updateArrows(int flag)
    {
        Arrow arr;
        double[][] table = graph;
        int k = 0;
        for (int i = 0; i < n; i++)
        {
            for (int j = 0; j < n; j++)
            {
                if (i == j) continue;
                arr = (Arrow) arrows.elementAt(k);
                switch (flag)
                {
                    case STANDARD:
                    case PREZERO:
                        arr.setStyle(0);
                        break;
                    case ZERO:
                        if (graph[i][j] == 0)
                        {
                            arr.setStyle(1);
                        }
                        else
                        {
                            arr.setStyle(0);
                        }
                        break;
                    case TRY_TREE:
                        if (par[j] == i)
                        {
                            arr.setStyle(2);
                        }
                        else if (graph[i][j] == 0)
                        {
                            arr.setStyle(1);
                        }
                        else
                        {
                            arr.setStyle(0);
                        }
                        break;
                    case BUILD_SCC:
                        if ((graph[i][j] == 0) && (scc[i] == scc[j]))
                        {
                            arr.setStyle(scc[i] + 5);
                        }
                        else
                        {
                            arr.setStyle(0);
                        }
                        break;
                    case NEW_GRAPH:
                        arr.setStyle(0);
                        break;
                    case PRE_BUILD:
                        table = tree;
                        arr.setStyle(0);
                        break;
                    case OPEN_COMPONENTS:
                        if (scc[i] == scc[j])
                        {
                            arr.setStyle(scc[i] + 5);
                        }
                        else
                        {
                            arr.setStyle(3);
                        }
                        table = tree;
                        break;
                    case PRE_TREE:
                        if (!Double.isNaN(tree[i][j]))
                        {
                            if (scc[i] != scc[j])
                            {
                                arr.setStyle(3);
                            }
                            else if ((scc[i] < comp) && (i == par[j]))
                            {
                                arr.setStyle(2);
                            }
                            else
                            {
                                arr.setStyle(scc[i] + 5);
                            }
                        }
                        table = tree;
                        break;
                    case BUILD_TREE:
                        arr.setStyle(2);
                        table = tree;
                        break;
                    case FINISH:
                        arr.setStyle(0);
                        table = tree;
                        break;
                    case ERROR:
                        break;
                }
                arr.setVisible(!Double.isNaN(table[i][j]));
                k++;
            }
        }
    }

    public void updateSizes(int top, int left, int width, int height)
    {
        this.top = top;
        this.left = left;
        this.width = width;
        this.height = height;
        asplit = (int) (width * split);
        graphCenter = new Point(left + asplit / 2, top + height / 2);
        graphWidth =  (int) ((graphCenter.x - left) * gRadius);
        graphHeight = (int) ((graphCenter.y - top) * gRadius);
        if (gAspectStatus)
        {
            int h = (int) (graphWidth * gAspect);
            if (h > graphHeight)
            {
                graphWidth = (int) (graphHeight / gAspect);
            }
            else
            {
                graphHeight = (int) (graphWidth * gAspect);
            }
        }
        vertexRadius = (int) (Math.min(graphWidth, graphHeight) * vRadius);
        int atWidth = (int) ((width - asplit) * tSize);
        int atHeight = (int) (height * tSize);
        if (tAspectStatus)
        {
            int h = (int) (atWidth * tAspect);
            if (h > atHeight)
            {
                atWidth = (int) (atHeight / tAspect);
            }
            else
            {
                atHeight = (int) (atWidth * tAspect);
            }
        }
        tableSize = new Dimension(atWidth, atHeight);
        tableXY = new Point(left + asplit + (width - asplit - atWidth) / 2, top + (height - atHeight) / 2);
        asplit += left;
        splitRect.setBounds(asplit - 2, 0, 4, height);
    }

    public void tableVisibility(boolean vis)
    {
        Rect rect;
        for (int i = 0; i < gtable.size(); i++)
        {
            rect = (Rect) gtable.elementAt(i);
            rect.setVisible(vis);
        }
    }

    public void setTableVisible(boolean vis)
    {
        splitRect.setVisible(vis);
        tableVisibility(vis);
        if (vis)
        {
            split = oldsplit;
            reshapeGraph();
        }
        else
        {
            oldsplit = split;
            split = 1;
            reshapeGraph();
        }
    }

    public Stack getTable()
    {
        return gtable;
    }

    public Stack getVertices()
    {
        return vertices;
    }

    public Stack getArrows()
    {
        return arrows;
    }

    public void setSplit(int asplit)
    {
        this.asplit = asplit;
        split = (double) (asplit  - left) / width;
        if (split > maxsplit)
        {
            split = maxsplit;
            this.asplit = left + (int) (width * maxsplit);
        }
        if (split < minsplit)
        {
            split = minsplit;
            this.asplit = left + (int) (width * minsplit);
        }
        reshapeGraph();
    }

    public int getSplit()
    {
        return asplit;
    }

    public Point[] getCoords()
    {
        Point[] pts = new Point[n];
        Ellipse circ;
        for (int i = 0; i < n; i++)
        {
            circ = (Ellipse) vertices.elementAt(i);
            pts[i] = circ.getLocation();
        }
        return pts;
    }

    public void setCoords(Point[] pts)
    {
        //update vertices
        Ellipse circ;
        for (int i = 0; i < n; i++)
        {
            circ = (Ellipse) vertices.elementAt(i);
            circ.setLocation(pts[i]);
        }

        // update arrows
        Arrow arr;
        int j = 0;
        for (int i = 0; i < n; i++)
        {
            for (int k = 0; k < n; k++)
            {
                if (i == k) continue;
                arr = (Arrow) arrows.elementAt(j);
                Point begin = new Point(pts[i].x + vertexRadius, pts[i].y + vertexRadius);
                Point end = new Point(pts[k].x + vertexRadius, pts[k].y + vertexRadius);
                double alpha = Math.atan2(end.y - begin.y, end.x - begin.x);
                begin.translate((int) (vertexRadius * Math.cos(alpha + arrowAngle)),
                                (int) (vertexRadius * Math.sin(alpha + arrowAngle)));
                end.translate((int) (vertexRadius * Math.cos(alpha - arrowAngle - Math.PI)),
                              (int) (vertexRadius * Math.sin(alpha - arrowAngle - Math.PI)));
                arr.setArrow(begin.x, begin.y, end.x, end.y);
                j++;
            }
        }
    }
}