package ru.ifmo.vizi.ctree.ui;

import ru.ifmo.vizi.base.ui.*;
import ru.ifmo.vizi.base.Configuration;

import java.awt.*;
import java.awt.event.ComponentEvent;
import java.awt.event.ItemEvent;
import java.util.Locale;

/**
 * Checkbox with a hint.
 *
 * @author  Svyatoslav Pimenov
 * @version $Id: 0.0$
 */
public abstract class HintedCheckbox extends Checkbox
 {
    /**
     * Creates a new hinted checkbox without caption and hint.
     */
    protected HintedCheckbox()
    {
        enableEvents(AWTEvent.ITEM_EVENT_MASK | AWTEvent.COMPONENT_EVENT_MASK);
    }

    /**
     * Creates a new hinted check with caption specified in parameter
     * <link>name</link> and hint specified in parameter <link>name</link>-hint.
     *
     * @param config applet configuration.
     * @param name checkbox name.
     */
    public HintedCheckbox(Configuration config, String name, Locale locale)
    {
        this();

        setLabel(config.getParameter(name + "-caption"));
        setHint(config.getParameter(name + "-hint"));
    }

    /**
     * Hint to show.
     */
    private String hint;

    /**
     * Sets new hint for this checkbox.
     *
     * @param hint hint to set.
     */
    protected void setHint(String hint)
    {
        this.hint = hint;
        applyHint();
    }

    /**
     * Applies hint, if needed.
     */
    private void applyHint()
    {
        if (isVisible())
        {
            Hinter.applyHint(this, hint);
        }
        else
        {
            Hinter.applyHint(this, null);
        }
    }

    /**
     * Invoked when an action occurs.
     *
     * @param e action event.
     */
    protected final void processItemEvent(ItemEvent e)
    {
        super.processItemEvent(e);
        click();
    }

    /**
     * Invoked when component state changes.
     *
     * @param e the component event
     */
    protected final void processComponentEvent(ComponentEvent e)
    {
        super.processComponentEvent(e);
        applyHint();
    }

    /**
     * Invoked when user has clicked on checkbox.
     */
    protected abstract void click();
}
