package ru.ifmo.vizi.ctree.widgets;

import ru.ifmo.vizi.base.widgets.*;
import ru.ifmo.vizi.base.widgets.Shape;

import java.awt.*;

/**
 * Arrow shape.
 * This shape may have border.
 *
 * @author  Svyatoslav Pimenov
 * @version $Id: 0.0$
 */
public final class Arrow extends Shape
{
    /**
     * half of arrow angle.
     */
    private final static double ARROW_ANGLE = Math.PI / 9;

    /**
     * length of arrow sides.
     */
    private final static int ARROW_LENGTH = 10;

    /**
     * Arrow's begin, end and points of arrow corner.
     */
    private Point begin;
    private Point end;
    private Point arr1;
    private Point arr2;

    /**
     * Creates a new arrow with specified style set, and direction.
     *
     * @param styleSet shape's style set.
     */
    public Arrow(ShapeStyle[] styleSet)
    {
        super(styleSet, "");
        begin = new Point(0, 0);
        end = new Point(0, 0);
        arr1 = new Point(0, 0);
        arr2 = new Point(0, 0);
    }

    /**
     * Returns minimum shape size to contain message with specified size.
     *
     * @param size message size.
     *
     * @return minimum shape size.
     */
    protected Dimension fit(Dimension size)
    {
        return new Dimension(size);
    }

    public void setArrow(int bx, int by, int ex, int ey)
    {

        begin.setLocation(bx, by);
        end.setLocation(ex, ey);
        double alpha = Math.atan2(begin.y - end.y, begin.x - end.x);
        arr1.setLocation(end.x + (int) (ARROW_LENGTH * Math.cos(alpha - ARROW_ANGLE)),
                             end.y + (int) (ARROW_LENGTH * Math.sin(alpha - ARROW_ANGLE)));
        arr2.setLocation(end.x + (int) (ARROW_LENGTH * Math.cos(alpha + ARROW_ANGLE)),
                             end.y + (int) (ARROW_LENGTH * Math.sin(alpha + ARROW_ANGLE)));

        Rectangle r = new Rectangle();
        r.setLocation(Math.min( Math.min(begin.x, end.x), Math.min(arr1.x, arr2.x)),
                      Math.min( Math.min(begin.y, end.y), Math.min(arr1.y, arr2.y)));
        r.setSize( Math.max (Math.max(begin.x, end.x), Math.max(arr1.x, arr2.x)) - r.x + 1,
                   Math.max (Math.max(begin.y, end.y), Math.max(arr1.y, arr2.y)) - r.y + 1);

        begin.translate(-r.x, -r.y);
        end.translate(-r.x, -r.y);
        arr1.translate(-r.x, -r.y);
        arr2.translate(-r.x, -r.y);

        setBounds(r);
    }

    public void setEnd(int ex, int ey)
    {
        Rectangle r = getBounds();
        Point b = new Point(begin.x + r.x, begin.y + r.y);
        setArrow(b.x, b.y, ex, ey);
    }

    /**
     * Paints this component.
     *
     * @param g graphics context for painting.
     */
    public void paint(Graphics g)
    {
        g.setColor(look.getBorderColor(style));
        g.drawLine(begin.x, begin.y, end.x, end.y);
        g.drawLine(end.x, end.y, arr1.x, arr1.y);
        g.drawLine(end.x, end.y, arr2.x, arr2.y);

        super.paint(g);
    }
}
